// src/nnn/is.ts
function is(type, arg) {
  return arg?.constructor === type;
}

// src/nnn/h.ts
var NS = {
  xlink: "http://www.w3.org/1999/xlink"
};
var _h = (namespaceURI) => {
  const createElement = namespaceURI == null ? (tag) => document.createElement(tag) : (tag) => document.createElementNS(namespaceURI, tag);
  const h = (tagOrNode, ...args) => {
    const node = is(String, tagOrNode) ? createElement(tagOrNode) : tagOrNode;
    args.forEach((arg) => {
      let child = null;
      if (arg instanceof Node) {
        child = arg;
      } else if (is(String, arg) || is(Number, arg)) {
        child = document.createTextNode(arg);
      } else if (is(Array, arg)) {
        child = h(...arg);
      } else if (arg != null) {
        for (const name in arg) {
          const value = arg[name];
          if (name[0] === "$") {
            const name1 = name.slice(1);
            if (is(Object, value)) {
              node[name1] = node[name1] ?? {};
              Object.assign(node[name1], value);
            } else {
              node[name1] = value;
            }
          } else if (node instanceof Element) {
            const indexOfColon = name.indexOf(":");
            if (indexOfColon >= 0) {
              const ns = NS[name.slice(0, indexOfColon)];
              if (ns != null) {
                const basename = name.slice(indexOfColon + 1);
                if (value === true) {
                  node.setAttributeNS(ns, basename, "");
                } else if (value === false) {
                  node.removeAttributeNS(ns, basename);
                } else {
                  node.setAttributeNS(ns, basename, is(String, value) ? value : "" + value);
                }
              }
            } else {
              if (value === true) {
                node.setAttribute(name, "");
              } else if (value === false) {
                node.removeAttribute(name);
              } else {
                node.setAttribute(name, is(String, value) ? value : "" + value);
              }
            }
          }
        }
      }
      if (child != null) {
        node.appendChild(child);
      }
    });
    return node;
  };
  return h;
};
var h = _h();
var s = _h("http://www.w3.org/2000/svg");

// src/nnn/chartable.ts
var COLORS = ["#e22", "#e73", "#fc3", "#ad4", "#4d9", "#3be", "#45d", "#c3e"];
var PADDING = 15;
var _svg = s("svg", { viewBox: "0 0 0 0", width: 0, height: 0 });
h(document.body, ["div", { $style: { width: 0, height: 0 } }, _svg]);
var toFixed2 = (value) => value.toFixed(2);
var chartable = ({
  table,
  headerColumn = false,
  xGap = 70,
  xLabelsMinHeight = 0,
  xLabelsRotate = false,
  xReverse = false,
  yGap = 30,
  yLabelsLeftMinWidth = 100,
  yLabelsRightMinWidth = 100,
  yMax = 10,
  zLabelsMinWidth = 0,
  zyMappings = []
}) => {
  const zxValues = [];
  const xLabels = [];
  const zLabels = [];
  table.querySelectorAll("tr").forEach((row, r) => row.querySelectorAll("td,th").forEach((col, c) => {
    const x = r - 1;
    const z = c - +headerColumn;
    const value = col.innerText;
    if (x >= 0 && z >= 0) {
      zxValues[z] = zxValues[z] ?? [];
      zxValues[z][x] = (zyMappings?.[z]?.[0] ?? parseFloat)(value);
    } else if (x >= 0 && z < 0) {
      xLabels[x] = value;
    } else if (x < 0 && z >= 0) {
      zLabels[z] = value;
    }
  }));
  if (xReverse) {
    xLabels.reverse();
    zxValues.forEach((xValues) => xValues.reverse());
  }
  const xMax = zxValues[0].length;
  const xPx = Array.from({ length: xMax }, (_, x) => x * xGap);
  const yPx = Array.from({ length: yMax }, (_, y) => y * yGap);
  const width = xGap * (xMax - 1);
  const height = yGap * (yMax - 1);
  const gGraph = s("g", ...yPx.map((px) => ["line", { x1: 0, x2: width, y1: px, y2: px, stroke: "#8888" }]), ...xPx.map((px) => ["line", { x1: px, x2: px, y1: 0, y2: height, stroke: "#8888" }]));
  const gXLabels = s("g");
  const gYLabelsL = zxValues.map((_) => s("g"));
  const gYLabelsR = zxValues.map((_) => s("g"));
  const gZColors = s("g");
  const gZLabels = [];
  const onclick = (z) => {
    gYLabelsL.forEach((g, z2) => s(g, { $style: { display: z2 === z ? "block" : "none" } }));
    gYLabelsR.forEach((g, z2) => s(g, { $style: { display: z2 === z ? "block" : "none" } }));
    gZLabels.forEach((text, z2) => s(text, { $style: { fontWeight: z2 === z ? "bold" : "normal", cursor: "pointer" } }));
  };
  zxValues.forEach((values, z) => {
    const fill = COLORS[z % COLORS.length];
    const minValue = Math.min(...values.filter((value) => !isNaN(value)));
    const maxValue = Math.max(...values.filter((value) => !isNaN(value)));
    yPx.forEach((yp, y) => {
      const value = (zyMappings?.[z]?.[1] ?? toFixed2)(maxValue - (maxValue - minValue) / yMax * (y + 1));
      const textL = s("text", { x: 0, y: yp, "text-anchor": "end", "alignment-baseline": "middle" }, value);
      const textR = s("text", { x: 0, y: yp, "text-anchor": "start", "alignment-baseline": "middle" }, value);
      s(_svg, textL, textR);
      yLabelsLeftMinWidth = Math.max(yLabelsLeftMinWidth, textL.getBBox().width);
      yLabelsRightMinWidth = Math.max(yLabelsRightMinWidth, textR.getBBox().width);
      s(gYLabelsL[z], textL);
      s(gYLabelsR[z], textR);
    });
    const text = s("text", {
      x: 0,
      y: yGap * z,
      "text-anchor": "start",
      "alignment-baseline": "middle",
      $onclick: () => onclick(z),
      $style: { fontWeight: "bold" }
    }, zLabels[z]);
    s(_svg, text);
    zLabelsMinWidth = Math.max(zLabelsMinWidth, text.getBBox().width);
    gZLabels[z] = text;
    s(gZColors, ["circle", { cx: 0, cy: yGap * z, r: 5, fill }]);
    const valuesPx = values.map((value) => isNaN(value) ? value : height * (maxValue - value) / (maxValue - minValue));
    xPx.forEach((px, x) => {
      if (!isNaN(valuesPx[x])) {
        s(gGraph, ["g", ["circle", { cx: px, cy: valuesPx[x], r: 5, fill }]]);
        if (!isNaN(valuesPx[x - 1])) {
          s(gGraph, ["line", { x1: px, x2: xPx[x - 1], y1: valuesPx[x], y2: valuesPx[x - 1], stroke: fill }]);
        }
      }
    });
  });
  if (xLabels.length > 0) {
    xPx.forEach((xp, x) => {
      const text = s("text", xLabelsRotate ? { x: 0, y: -xp, "text-anchor": "start", "alignment-baseline": "middle", transform: "rotate(90)" } : { x: xp, y: PADDING, "text-anchor": "middle", "alignment-baseline": "middle" }, xLabels[x]);
      s(_svg, text);
      xLabelsMinHeight = Math.max(xLabelsMinHeight, xLabelsRotate ? text.getBBox().width : text.getBBox().height);
      s(gXLabels, text);
    });
  }
  const svgW = width + yLabelsLeftMinWidth + yLabelsRightMinWidth + zLabelsMinWidth + 4 * PADDING;
  const svgH = height + xLabelsMinHeight + (xLabels.length > 0 ? 3 : 2) * PADDING;
  s(gGraph, { transform: `translate(${yLabelsLeftMinWidth + PADDING} ${PADDING})` });
  gYLabelsL.forEach((g) => s(g, { transform: `translate(${yLabelsLeftMinWidth} ${PADDING})` }));
  gYLabelsR.forEach((g) => s(g, { transform: `translate(${width + yLabelsLeftMinWidth + 2 * PADDING} ${PADDING})` }));
  s(gZColors, { transform: `translate(${width + yLabelsLeftMinWidth + yLabelsRightMinWidth + 3 * PADDING} ${PADDING})` });
  s(gXLabels, { transform: `translate(${yLabelsLeftMinWidth + PADDING} ${height + 2 * PADDING})` });
  onclick(0);
  return s("svg", { viewBox: `0 0 ${svgW} ${svgH}`, width: `${svgW}px`, height: `${svgH}px`, class: "chartable" }, gGraph, gXLabels, ...gYLabelsL, ...gYLabelsR, gZColors, ["g", { transform: `translate(${width + yLabelsLeftMinWidth + yLabelsRightMinWidth + 4 * PADDING} ${PADDING})` }, ...gZLabels]);
};

// src/nnn/eq.ts
var eq = (x, y) => {
  if (x === y) {
    return true;
  }
  const xConstructor = x?.constructor;
  if (xConstructor === y?.constructor) {
    if (xConstructor === Number) {
      return isNaN(x) && isNaN(y) || +x === +y;
    }
    if (xConstructor === Date) {
      return +x === +y;
    }
    if (xConstructor === String || xConstructor === RegExp) {
      return "" + x === "" + y;
    }
    if (xConstructor === Array) {
      return x.length === y.length && x.every((item, index) => eq(item, y[index]));
    }
    if (xConstructor === Object) {
      const keysOfX = Object.keys(x);
      return keysOfX.length === Object.keys(y).length && keysOfX.every((key) => eq(x[key], y[key]));
    }
    if (xConstructor === Set || xConstructor === Map) {
      if (x.size !== y.size) {
        return false;
      }
      const xa = [...x];
      const ya = [...y];
      return xa.every((xv) => ya.find((yv) => eq(xv, yv)));
    }
  }
  return false;
};

// src/nnn/escape.ts
var escapeValues = (escapeMap, values) => values.map((value) => (escapeMap.get(value?.constructor) ?? escapeMap.get(undefined))?.(value) ?? "");
var escape = (escapeMap, template, ...values) => String.raw(template, ...escapeValues(escapeMap, values));

// src/nnn/has.ts
var has = (key, ref) => (is(String, key) || is(Number, key) || is(Symbol, key)) && Object.hasOwnProperty.call(ref ?? Object, key);

// src/nnn/fixTypography.ts
var TAGS_TO_SKIP = { IFRAME: 1, NOSCRIPT: 1, PRE: 1, SCRIPT: 1, STYLE: 1, TEXTAREA: 1 };
var fixTypography = (node) => {
  const queue = [node];
  while (queue.length > 0) {
    const node0 = queue.shift();
    if (node0 instanceof Element) {
      node0.childNodes.forEach((childNode) => {
        if (childNode instanceof Text) {
          queue.push(childNode);
        } else if (childNode instanceof Element && !has(childNode.tagName, TAGS_TO_SKIP)) {
          queue.push(childNode);
        }
      });
    } else if (node0 instanceof Text) {
      const nodeValue = node0.nodeValue?.trim?.();
      if (nodeValue != null) {
        let previousNode = node0;
        nodeValue.split(/(\s|\(|„)([aiouwz—]\s)/gi).forEach((chunk, i) => {
          i %= 3;
          const currentNode = i === 2 ? h("span", { style: "white-space:nowrap" }, chunk) : i === 1 ? document.createTextNode(chunk) : document.createTextNode(chunk.replace(/(\/(?=[^/\s])|\.(?=[^\s]))/g, "$1\u200B"));
          if (node0.parentNode != null) {
            node0.parentNode.insertBefore(currentNode, previousNode.nextSibling);
          }
          previousNode = currentNode;
        });
        node0.parentNode?.removeChild(node0);
      }
    }
  }
};

// src/nnn/jcss.ts
var _jcss = (node, prefix, result, split) => {
  const queue = [[node, prefix]];
  while (queue.length > 0) {
    const [style0, prefix0] = queue.shift() ?? [];
    if (style0 == null || prefix0 == null) {
      continue;
    }
    if (is(Array, style0)) {
      result.push(prefix0, prefix0 !== "" ? "{" : "", style0.join(";"), prefix0 !== "" ? "}" : "");
    } else {
      const todo = [];
      let attributes = [];
      let attributesPushed = false;
      for (const key in style0) {
        const value = style0[key];
        if (is(String, value) || is(Number, value)) {
          if (!attributesPushed) {
            attributesPushed = true;
            attributes = [];
            todo.push([attributes, prefix0]);
          }
          attributes.push(`${split(key).replace(/([A-Z])/g, (_, letter) => "-" + letter.toLowerCase())}:${value}`);
        } else if (value != null) {
          attributesPushed = false;
          const prefixN = [];
          const keyChunks = key.split(",");
          prefix0.split(",").forEach((prefixChunk) => keyChunks.forEach((keyChunk) => prefixN.push(prefixChunk + keyChunk)));
          todo.push([value, prefixN.join(",")]);
        }
      }
      queue.unshift(...todo);
    }
  }
};
var jcss = (root, splitter = "$$") => {
  const split = (text) => text.split(splitter)[0];
  const chunks = [];
  for (const key in root) {
    const value = root[key];
    if (value != null) {
      if (key[0] === "@") {
        chunks.push(split(key) + "{");
        _jcss(value, "", chunks, split);
        chunks.push("}");
      } else {
        _jcss(value, split(key), chunks, split);
      }
    }
  }
  return chunks.join("");
};

// src/nnn/jsOnParse.ts
var jsOnParse = (handlers, text) => JSON.parse(text, (key, value) => {
  if (is(Object, value)) {
    let isSecondKey = false;
    for (key in value) {
      if (isSecondKey) {
        return value;
      }
      isSecondKey = true;
    }
    if (has(key, handlers) && is(Array, value[key])) {
      return handlers[key](...value[key]);
    }
  }
  return value;
});

// src/nnn/locale.ts
var locale = (map, defaultVersion) => (text, version = defaultVersion) => {
  const textV = map?.[version]?.[text];
  const textD = map?.[defaultVersion]?.[text];
  return is(String, textV) ? textV : is(String, textD) ? textD : text;
};

// src/nnn/nanolight.ts
var nanolight = (pattern, highlighters, code) => {
  const result = [];
  code.split(pattern).forEach((chunk, index) => {
    if (chunk != null && chunk !== "") {
      index %= highlighters.length;
      result.push(highlighters[index](chunk, index));
    }
  });
  return result;
};

// src/nnn/nanolightJs.ts
var nanolightJs = nanolight.bind(0, /('.*?'|".*?"|`[\s\S]*?`)|(\/\/.*?\n|\/\*[\s\S]*?\*\/)|(any|bigint|break|boolean|case|catch|const|continue|debugger|default|delete|do|else|eval|export|extends|false|finally|for|from|function|goto|if|import|in|instanceof|is|keyof|let|NaN|new|number|null|package|return|string|super|switch|symbol|this|throw|true|try|type|typeof|undefined|unknown|var|void|while|with|yield)(?!\w)|([<>=.?:&|!^~*/%+-])|(0x[\dabcdef]+|0o[01234567]+|0b[01]+|\d+(?:\.[\d_]+)?(?:e[+-]?[\d_]+)?)|([$\w]+)(?=\()|([$\wąćęłńóśżźĄĆĘŁŃÓŚŻŹ]+)/, [
  (chunk) => chunk,
  (chunk) => ["span", { class: "string" }, chunk],
  (chunk) => ["span", { class: "comment" }, chunk],
  (chunk) => ["span", { class: "keyword" }, chunk],
  (chunk) => ["span", { class: "operator" }, chunk],
  (chunk) => ["span", { class: "number" }, chunk],
  (chunk) => ["span", { class: "function" }, chunk],
  (chunk) => ["span", { class: "literal" }, chunk]
]);

// src/nnn/pick.ts
var pick = (obj, keys) => Object.fromEntries(Object.entries(obj).filter(([key]) => keys.includes(key)));
var omit = (obj, keys) => Object.fromEntries(Object.entries(obj).filter(([key]) => !keys.includes(key)));

// src/nnn/plUral.ts
var plUral = (singular, plural2, plural5, value) => {
  const absValue = Math.abs(value);
  const absValueMod10 = absValue % 10;
  return value === 1 ? singular : (absValueMod10 === 2 || absValueMod10 === 3 || absValueMod10 === 4) && absValue !== 12 && absValue !== 13 && absValue !== 14 ? plural2 : plural5;
};

// src/nnn/pro.ts
var pro = (ref) => new Proxy(ref, {
  get(target, key) {
    return pro(target[key] = target[key] ?? {});
  }
});

// src/nnn/refsInfo.ts
var refsInfo = (...refs) => {
  const fns = new Set;
  refs.forEach((ref) => {
    while (is(Function, ref) && !fns.has(ref) && `${ref}`.includes("[native code]")) {
      fns.add(ref);
      ref = Object.getPrototypeOf(ref);
    }
  });
  return Array.from(fns.values()).map((fn) => [
    fn.name,
    Object.getPrototypeOf(fn)?.name ?? "",
    Object.getOwnPropertyNames(fn.prototype ?? Object.create(null)).sort()
  ]).sort((a, b) => -(a[0] < b[0]));
};

// src/nnn/uuid1.ts
var ZEROS = "0".repeat(16);
var counter = 0;
var uuid1 = ({
  date = new Date,
  node = Math.random().toString(16).slice(2)
} = {}) => {
  const time = ZEROS + (1e4 * (+date + 12219292800000)).toString(16);
  counter = counter + 1 & 16383;
  return time.slice(-8).concat("-", time.slice(-12, -8), -1, time.slice(-15, -12), "-", (8 | counter >> 12).toString(16), (ZEROS + (counter & 4095).toString(16)).slice(-3), "-", (ZEROS + node).slice(-12));
};
export {
  uuid1,
  s,
  refsInfo,
  pro,
  plUral,
  pick,
  omit,
  nanolightJs,
  nanolight,
  locale,
  jsOnParse,
  jcss,
  is,
  has,
  h,
  fixTypography,
  escapeValues,
  escape,
  eq,
  chartable
};
